@@ -50,13 +50,10 @@ public:
   {
   }
 
-  predicate_UsesRelationshipHasRelationships(const sharedptr<const UsesRelationship> uses_relationship_name, bool first_level_only = false)
+  predicate_UsesRelationshipHasRelationships(const sharedptr<const UsesRelationship> uses_relationship_name)
   : m_relationship_name(uses_relationship_name->get_relationship_name()),
     m_related_relationship_name(uses_relationship_name->get_related_relationship_name())
   {
-    //If first_level_only, search for relationships that have the same top-level relationship, but have no related relationship.
-    if(first_level_only)
-      m_related_relationship_name = Glib::ustring();
   }
 
   virtual ~predicate_UsesRelationshipHasRelationships()
@@ -188,36 +185,6 @@ Glib::ustring Utils::build_sql_select_with_where_clause(const Glib::ustring& tab
   return build_sql_select_with_where_clause(table_name, constFieldsToGet, where_clause, sort_clause);
 }
 
-
-static typedef std::list< sharedptr<const UsesRelationship> > type_list_relationships;
-
-static void add_to_relationships_list(type_list_relationships& list_relationships, const sharedptr<const LayoutItem_Field>& layout_item)
-{
-  if(!(layout_item->get_has_relationship_name()))
-    return;
-
-  //If this is a related relationship, add the first-level relationship too, so that the related relationship can be defined in terms of it:
-  type_list_relationships::const_iterator iterFind = std::find_if(list_relationships.begin(), list_relationships.end(), predicate_UsesRelationshipHasRelationships<UsesRelationship>(layout_item, true /* top_level_only */) );
-  if(iterFind == list_relationships.end()) //If the table is not yet in the list:
-  {
-    sharedptr<UsesRelationship> uses_rel = sharedptr<UsesRelationship>::create();
-    uses_rel->set_relationship(layout_item->get_relationship());
-    list_relationships.push_front(uses_rel); //These need to be at the front, so that related relationships can use them later in the SQL statement.
-  }
-
-  //Add the relationship to the list:
-  iterFind = std::find_if(list_relationships.begin(), list_relationships.end(), predicate_UsesRelationshipHasRelationships<UsesRelationship>(layout_item) );
-  if(iterFind == list_relationships.end()) //If the table is not yet in the list:
-  {
-    sharedptr<UsesRelationship> uses_rel = sharedptr<UsesRelationship>::create();
-    uses_rel->set_relationship(layout_item->get_relationship());
-    uses_rel->set_related_relationship(layout_item->get_related_relationship());
-    list_relationships.push_back(uses_rel);
-  }
-
- 
-}
-
 Glib::ustring Utils::build_sql_select_with_where_clause(const Glib::ustring& table_name, const type_vecConstLayoutFields& fieldsToGet, const Glib::ustring& where_clause, const type_sort_clause& sort_clause)
 {
   Glib::ustring result;
@@ -229,16 +196,41 @@ Glib::ustring Utils::build_sql_select_with_where_clause(const Glib::ustring& tab
   for(type_vecConstLayoutFields::const_iterator iter = fieldsToGet.begin(); iter != fieldsToGet.end(); ++iter)
   {
     sharedptr<const LayoutItem_Field> layout_item = *iter;
-    add_to_relationships_list(list_relationships, layout_item);
+
+    if(layout_item->get_has_relationship_name())
+    {
+      //Add the relationship to the list:
+      type_list_relationships::const_iterator iterFind = std::find_if(list_relationships.begin(), list_relationships.end(), predicate_UsesRelationshipHasRelationships<UsesRelationship>(layout_item) );
+      if(iterFind == list_relationships.end()) //If the table is not yet in the list:
+      {
+        sharedptr<UsesRelationship> uses_rel = sharedptr<UsesRelationship>::create();
+        uses_rel->set_relationship(layout_item->get_relationship());
+        uses_rel->set_related_relationship(layout_item->get_related_relationship());
+        list_relationships.push_back(uses_rel);
+      }
+    }
   }
 
   for(type_sort_clause::const_iterator iter = sort_clause.begin(); iter != sort_clause.end(); ++iter)
   {
     sharedptr<const LayoutItem_Field> layout_item = iter->first;
-    add_to_relationships_list(list_relationships, layout_item);
+
+    if(layout_item->get_has_relationship_name())
+    {
+      //Add the relationship to the list:
+      type_list_relationships::const_iterator iterFind = std::find_if(list_relationships.begin(), list_relationships.end(), predicate_UsesRelationshipHasRelationships<UsesRelationship>(layout_item) );
+      if(iterFind == list_relationships.end()) //If the table is not yet in the list:
+      {
+        sharedptr<UsesRelationship> uses_rel = sharedptr<UsesRelationship>::create();
+        uses_rel->set_relationship(layout_item->get_relationship());
+        uses_rel->set_related_relationship(layout_item->get_related_relationship());
+        list_relationships.push_back(uses_rel);
+      }
+    }
   }
 
 
+
   Glib::ustring sql_part_fields;
   Glib::ustring sql_part_from;
 
@@ -379,7 +371,10 @@ Utils::type_list_values_with_second Utils::get_choice_values(const sharedptr<con
   Glib::ustring choice_field, choice_second;
   field->get_formatting_used().get_choices(choice_relationship, choice_field, choice_second);
   if(!choice_relationship)
+  {
+    //std::cout <<" debug: field has no choices: " << field->get_name() << std::endl;
     return list_values;
+  }
 
   const Glib::ustring to_table = choice_relationship->get_to_table();
   if(to_table.empty())
@@ -398,6 +393,7 @@ Utils::type_list_values_with_second Utils::get_choice_values(const sharedptr<con
 
   sql_query += " FROM \"" + choice_relationship->get_to_table() + "\" ORDER BY \"" + to_table + "\".\"" + choice_field + "\"";
 
+  //std::cout << "debug: get_choice_values(): query: " << sql_query << std::endl;
   //Connect to database:
   sharedptr<SharedConnection> connection = ConnectionPool::get_instance()->connect();
 
@@ -407,14 +403,15 @@ Utils::type_list_values_with_second Utils::get_choice_values(const sharedptr<con
 
   if(datamodel)
   {
-    guint count = datamodel->get_n_rows();
+    const guint count = datamodel->get_n_rows();
+    //std::cout << "  result: count=" << count << std::endl;
     for(guint row = 0; row < count; ++row)
     {
       std::pair<Gnome::Gda::Value, Gnome::Gda::Value> itempair;
       itempair.first = datamodel->get_value_at(0, row);
 
       if(with_second)
-        itempair.second = datamodel->get_value_at(1, row);
+			itempair.second = datamodel->get_value_at(1, row);
 
       list_values.push_back(itempair);
     }
@@ -588,6 +585,8 @@ Glib::ustring Utils::title_from_string(const Glib::ustring& text)
 
 Utils::type_vecStrings Utils::string_separate(const Glib::ustring& str, const Glib::ustring& separator, bool ignore_quoted_separator)
 {
+  //std::cout << "Utils::string_separate(): separator=" << separator << std::endl;
+
   type_vecStrings result;
 
   const Glib::ustring::size_type size = str.size();
@@ -610,7 +609,7 @@ Utils::type_vecStrings Utils::string_separate(const Glib::ustring& str, const Gl
       //Check that the separator was not in quotes:
       bool in_quotes = false;
 
-      if(ignore_quoted_separator)
+      if(!ignore_quoted_separator)
       {
         //std::cout << "  debug: attempting to ignore quoted separators: " << separator << std::endl;
        
@@ -688,7 +687,7 @@ Utils::type_vecStrings Utils::string_separate(const Glib::ustring& str, const Gl
 
         //Store this item, and start the next item after it:
         item = str.substr(item_start, posComma - item_start);
-        //std::cout << "ITEM. pos_comma=" << posComma << ", ITEM= " << item << std::endl;
+        //std::cout << "  ITEM. pos_comma=" << posComma << ", ITEM= " << item << std::endl;
 	item_start = posComma + size_separator;
       }
       //else
